/*
 * Copyright (C) 2009   Thomas Chou <thomas@wytron.com.tw>
 * Copyright (C) 2004   Microtronix Datacom Ltd
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#ifndef _ASM_NIOS2_BYTEORDER_H
#define _ASM_NIOS2_BYTEORDER_H

#include <linux/byteorder/little_endian.h>

#endif
